﻿using System;
using Ozeki.Network;
using Ozeki.VoIP;

namespace CustomAuth
{
    class MyPBX : PBXBase
    {
        string localAddress;
        UserInfoContainer userInfoContainer;

        public MyPBX(string localAddress, int minPortRange, int maxPortRange) : base(minPortRange, maxPortRange)
        {
            this.localAddress = localAddress;
            Console.WriteLine("PBX starting...");
            Console.WriteLine("Local address: " + localAddress);

            SetUserInfos();
        }

        protected override void OnStart()
        {
            SetListenPort(localAddress, 5060, TransportType.Udp);
            Console.WriteLine("Listened port: 5060(UDP)");

            Console.WriteLine("PBX started.");
            base.OnStart();
        }

        protected override AuthenticationResult OnAuthenticationRequest(ISIPExtension extension, RequestAuthenticationInfo authInfo)
        {
            Console.WriteLine("Authentication request received from: " + extension.ExtensionID);

            AuthenticationResult result = new AuthenticationResult();

            UserInfo userInfo;
            if (userInfoContainer.TryGetUserInfo(extension.ExtensionID, out userInfo))
            {
                result = extension.CheckPassword(userInfo.UserName, userInfo.Password, authInfo);
            }
            else
            {
                Console.WriteLine("Cannot find extension. UserName: " + extension.ExtensionID);
            }

            if (result.AuthenticationAccepted)
            {
                Console.WriteLine("Authentication accepted. UserName: " + extension.ExtensionID);
            }
            else
            {
                Console.WriteLine("Authentication failed. UserName: " + extension.ExtensionID);
            }

            return result;
        }

        protected override RegisterResult OnRegisterReceived(ISIPExtension extension, SIPAddress from, int expires)
        {
            Console.WriteLine("Register received from: " + extension.ExtensionID);
            return base.OnRegisterReceived(extension, from, expires);
        }

        protected override void OnUnregisterReceived(ISIPExtension extension)
        {
            Console.WriteLine("Unregister received from: " + extension.ExtensionID);
            base.OnUnregisterReceived(extension);
        }

        protected override void OnCallRequestReceived(ISessionCall call)
        {
            Console.WriteLine("Call request received. Caller: " + call.DialInfo.CallerID + " callee: " + call.DialInfo.Dialed);
            base.OnCallRequestReceived(call);
        }

        private void SetUserInfos()
        {
            userInfoContainer = new UserInfoContainer();

            for (int i = 0; i < 10; i++)
            {
                var userName = string.Format("10{0}", i);
                var password = userName;

                userInfoContainer.AddUserInfo(new UserInfo(userName, password));
            }
        }
    }
}
